﻿using System;
using System.Collections;
using HeuristicLab.Persistence.Interfaces;

namespace HeuristicLab.Persistence.Default.Decomposers {
  
  public class ArrayDecomposer : IDecomposer {

    public bool CanDecompose(Type type) {
      return type.IsArray || type == typeof(Array);
    }

    public IEnumerable DeCompose(object array) {
      Array a = (Array)array;
      yield return a.Rank;
      for (int i = 0; i < a.Rank; i++) {
        yield return a.GetLength(i);
      }
      foreach (object o in (Array)array) {
        yield return o;
      }
    }

    public object Compose(IEnumerable elements, Type t) {
      IEnumerator e = elements.GetEnumerator();
      e.MoveNext();
      int rank = (int)e.Current;
      int[] lengths = new int[rank];
      for (int i = 0; i < rank; i++) {
        e.MoveNext();
        lengths[i] = (int)e.Current;
      }
      Array a = Array.CreateInstance(t.GetElementType(), lengths);      
      int[] positions = new int[rank];
      while (e.MoveNext()) {
        a.SetValue(e.Current, positions);
        positions[0] += 1;
        for (int i = 0; i < rank-1; i++) {
          if (positions[i] >= lengths[i]) {
            positions[i] = 0;
            positions[i + 1] += 1;
          } else {
            break;
          }
        }
      }
      return a;
    }
  }
  
}
