﻿using System;
using System.Collections;
using System.Reflection;
using HeuristicLab.Persistence.Interfaces;

namespace HeuristicLab.Persistence.Default.Decomposers {
  
  public class EnumerableDecomposer : IDecomposer {

    public bool CanDecompose(Type type) {
      return
        type.GetInterface(typeof(IEnumerable).FullName) != null &&
        type.GetMethod("Add") != null &&
        type.GetMethod("Add").GetParameters().Length == 1 &&
        type.GetConstructor(
          BindingFlags.Public |
          BindingFlags.NonPublic |
          BindingFlags.Instance,
          null, Type.EmptyTypes, null) != null;        
    }    

    public IEnumerable DeCompose(object obj) {
      return (IEnumerable)obj;
    }

    public object Compose(IEnumerable objects, Type type) {
      object instance = Activator.CreateInstance(type, true);
      MethodInfo addMethod = type.GetMethod("Add");
      foreach (object o in objects) {
        addMethod.Invoke(instance, new[] {o});        
      }
      return instance;
    }

  }

}
